/* $Id: BuildL1CrisSubset.c,v 1.1 1998/01/26 06:24:41 rgr Exp $ */

/* $RCSfile: BuildL1CrisSubset.c,v $ */
/* $Author: rgr $ */
/* $Date: 1998/01/26 06:24:41 $ */
/* $Revision: 1.1 $ */

#include <sys/types.h>
#include <memory.h>

#include "AceConstants.h"
#include "CrisConstants.h"

#include "cris/L1CrisSubset.h"
#include "downlink/AceBlock.h"
#include "downlink/UnpackedDddHeader.h"
#include "downlink/UnpackedPacketHeader.h"
#include "downlink/UnpackedMinorFrameHeader.h"

#include "UnpackDddHeader.h"
#include "UnpackPacketHeader.h"
#include "UnpackMinorFrameHeader.h"
#include "ace/AceEpochTime.h"
#include "ace/cvtToSec1996_0.h"
#include "ace/cvtToSec1996_yd_0.h"


void BuildL1CrisSubset(AceBlock *block, struct L1CrisSubset *subset)
{
   UnpackedDddHeader         DddHeader;
   UnpackedPacketHeader      PacketHeader;
   UnpackedMinorFrameHeader  MinorFrameHeader;

   static u_long LaunchDate = 0L;

   if (!LaunchDate)
       LaunchDate = cvtToSec1996_0(1997L, 8L, 25L, 0L, 0L, 0L);

   UnpackDddHeader(block->DddHeader, &DddHeader);
   subset->microsecond = ((u_long) (DddHeader.TimeOfDay % 100))*10000L;
   subset->Second1996 = cvtToSec1996_yd_0((long) DddHeader.year,
                                         (double) DddHeader.DayOfYear);
   subset->Second1996 += (u_long) DddHeader.TimeOfDay / 100L;
   
   UnpackPacketHeader(block->telemetry.PacketHeader, &PacketHeader);
   subset->ClockMinorFrame = PacketHeader.PacketSecondaryHeader;
   if (subset->Second1996 > LaunchDate) 
      AceEpochTime(subset->ClockMinorFrame,
                   &(subset->Second1996),
                   &(subset->microsecond));

   UnpackMinorFrameHeader(block->telemetry.MinorFrameHeader,
                                                    &MinorFrameHeader);
   subset->MajorFrameCount = MinorFrameHeader.MajorFrameCount;
   subset->MinorFrameCount = MinorFrameHeader.MinorFrameCount;
   subset->FormatId = MinorFrameHeader.FormatId;
  
   subset->quality = 0x00;

   switch (subset->FormatId)
   {
       case DOWNLINK_FORMAT_SCIENCE:
          if (subset->MinorFrameCount == 3)
          {
             subset->MainBusVoltage = block->telemetry.data[14];
             subset->CrisInternalTemperature1 = 0;
             subset->CrisInternalTemperature2 = 0;
             subset->CrisInterfaceTemperature = 0;
             subset->CrisCurrent = 0;
          }
          else if (subset->MinorFrameCount == 9)
          {
             subset->MainBusVoltage = 0;
             subset->CrisInternalTemperature1 =
                                        block->telemetry.data[17];
             subset->CrisInternalTemperature2 =
                                        block->telemetry.data[18];
             subset->CrisInterfaceTemperature = 0;
             subset->CrisCurrent = 0;
          }
          else if (subset->MinorFrameCount == 10)
          {
             subset->MainBusVoltage = 0;
             subset->CrisCurrent = block->telemetry.data[6];
             subset->CrisInternalTemperature1 = 0;
             subset->CrisInternalTemperature2 = 0;
             subset->CrisInterfaceTemperature = 0;
          }
          else
          {
             subset->MainBusVoltage = 0;
             subset->CrisInternalTemperature1 = 0;
             subset->CrisInternalTemperature2 = 0;
             subset->CrisInterfaceTemperature = 0;
             subset->CrisCurrent = 0;
          }
          break;
       case DOWNLINK_FORMAT_ADC:
          subset->MainBusVoltage           = block->telemetry.data[508];
          subset->CrisInternalTemperature1 = block->telemetry.data[628];
          subset->CrisInternalTemperature2 = block->telemetry.data[629];
          subset->CrisInterfaceTemperature = block->telemetry.data[772];
          subset->CrisCurrent = block->telemetry.data[635];
          break;
       case DOWNLINK_FORMAT_LOW_RATE_HSKP:
       case DOWNLINK_FORMAT_LOW_RATE_CDH_MEMORY:
       case DOWNLINK_FORMAT_LOW_RATE_CDH_BIN:
          subset->MainBusVoltage = block->telemetry.data[71];
          subset->CrisInternalTemperature1 = block->telemetry.data[191];
          subset->CrisInternalTemperature2 = block->telemetry.data[192];
          subset->CrisInterfaceTemperature = block->telemetry.data[335];
          subset->CrisCurrent = block->telemetry.data[198];
          break;
       case DOWNLINK_FORMAT_LOW_RATE_ADC:
          subset->MainBusVoltage = block->telemetry.data[68];
          subset->CrisInternalTemperature1 =
                                       block->telemetry.data[188];
          subset->CrisInternalTemperature2 =
                                       block->telemetry.data[189];
          subset->CrisCurrent = block->telemetry.data[195];
          subset->CrisInterfaceTemperature = 0;
          break;
      case DOWNLINK_FORMAT_RTSW:
          subset->MainBusVoltage = block->telemetry.data[810];
          subset->CrisInternalTemperature1 = 0;
          subset->CrisInternalTemperature2 = 0;
          subset->CrisInterfaceTemperature = 0;
          subset->CrisCurrent = 0;
          break;
      default:
          subset->MainBusVoltage = 0;
          subset->CrisInternalTemperature1 = 0;
          subset->CrisInternalTemperature2 = 0;
          subset->CrisInterfaceTemperature = 0;
          subset->CrisCurrent = 0;
          break;
   }

   switch (subset->FormatId)
   {
       case DOWNLINK_FORMAT_SCIENCE:
       case DOWNLINK_FORMAT_ADC:
       case DOWNLINK_FORMAT_CDH_MEMORY:
       case DOWNLINK_FORMAT_CDH_BIN:
          memcpy(subset->CrisData,
                 block->telemetry.data + TELEMETRY_OFFSET_CRIS,
                 NUM_CRIS_TELEMETRY_BYTES);
          break;
      default:
          memset(subset->CrisData, 0, NUM_CRIS_TELEMETRY_BYTES);
          break;
   }
}
