/* $Id: BuildL1SisSubset.c,v 1.1 1998/01/26 06:24:58 rgr Exp $ */

/* $RCSfile: BuildL1SisSubset.c,v $ */
/* $Author: rgr $ */
/* $Date: 1998/01/26 06:24:58 $ */
/* $Revision: 1.1 $ */

#include <sys/types.h>
#include <memory.h>

#include "AceConstants.h"
#include "SisConstants.h"

#include "sis/L1SisSubset.h"
#include "downlink/AceBlock.h"
#include "downlink/UnpackedDddHeader.h"
#include "downlink/UnpackedPacketHeader.h"
#include "downlink/UnpackedMinorFrameHeader.h"

#include "UnpackDddHeader.h"
#include "UnpackPacketHeader.h"
#include "UnpackMinorFrameHeader.h"
#include "ace/AceEpochTime.h"
#include "ace/cvtToSec1996_0.h"
#include "ace/cvtToSec1996_yd_0.h"


void BuildL1SisSubset(AceBlock *block, struct L1SisSubset *subset)
{
   UnpackedDddHeader         DddHeader;
   UnpackedPacketHeader      PacketHeader;
   UnpackedMinorFrameHeader  MinorFrameHeader;

   static u_long LaunchDate = 0L;

   if (!LaunchDate)
       LaunchDate = cvtToSec1996_0(1997L, 8L, 25L, 0L, 0L, 0L);

   UnpackDddHeader(block->DddHeader, &DddHeader);
   subset->microsecond = ((u_long) (DddHeader.TimeOfDay % 100))*10000L;
   subset->Second1996 = cvtToSec1996_yd_0((long) DddHeader.year,
                                         (double) DddHeader.DayOfYear);
   subset->Second1996 += (u_long) DddHeader.TimeOfDay / 100L;
   
   UnpackPacketHeader(block->telemetry.PacketHeader, &PacketHeader);
   subset->ClockMinorFrame = PacketHeader.PacketSecondaryHeader;
   if (subset->Second1996 > LaunchDate) 
      AceEpochTime(subset->ClockMinorFrame,
                   &(subset->Second1996),
                   &(subset->microsecond));

   UnpackMinorFrameHeader(block->telemetry.MinorFrameHeader,
                                                    &MinorFrameHeader);
   subset->MajorFrameCount = MinorFrameHeader.MajorFrameCount;
   subset->MinorFrameCount = MinorFrameHeader.MinorFrameCount;
   subset->FormatId = MinorFrameHeader.FormatId;

   subset->quality = 0x00;
  
   switch (subset->FormatId)
   {
       case DOWNLINK_FORMAT_SCIENCE:
          if (subset->MinorFrameCount == 3)
          {
             subset->MainBusVoltage = block->telemetry.data[14];
             subset->SisInternalTemperature1  = 0;
             subset->SisInternalTemperature2  = 0;
             subset->SisInterfaceTemperature  = 0;
             subset->SisCurrent  = 0;
             subset->SisCoverTelltale = 255;
          }
          else if (subset->MinorFrameCount == 9)
          {
             subset->MainBusVoltage = 0;
             subset->SisInternalTemperature1 = 
                                        block->telemetry.data[19];
             subset->SisInternalTemperature2 =
                                        block->telemetry.data[20];
             subset->SisInterfaceTemperature = 0;
             subset->SisCurrent  = 0;
             subset->SisCoverTelltale = 255;
          }
          else if (subset->MinorFrameCount == 10)
          {
             subset->MainBusVoltage = 0;
             subset->SisCoverTelltale =
                        (block->telemetry.data[5] & 0x40) ? 1 : 0;
             subset->SisCurrent = block->telemetry.data[7];
             subset->SisInternalTemperature1  = 0;
             subset->SisInternalTemperature2  = 0;
             subset->SisInterfaceTemperature  = 0;
          }
          else
          {
             subset->MainBusVoltage = 0;
             subset->SisInternalTemperature1  = 0;
             subset->SisInternalTemperature2  = 0;
             subset->SisInterfaceTemperature  = 0;
             subset->SisCurrent  = 0;
             subset->SisCoverTelltale = 255;
          }
          break;
       case DOWNLINK_FORMAT_ADC:
          subset->MainBusVoltage           = block->telemetry.data[508];
          subset->SisInternalTemperature1  = block->telemetry.data[630];
          subset->SisInternalTemperature2  = block->telemetry.data[631];
          subset->SisInterfaceTemperature  = block->telemetry.data[773];
          subset->SisCoverTelltale =
                            (block->telemetry.data[634] & 0x40) ? 1 : 0;
          subset->SisCurrent  = block->telemetry.data[636];
          break;
       case DOWNLINK_FORMAT_LOW_RATE_HSKP:
       case DOWNLINK_FORMAT_LOW_RATE_CDH_MEMORY:
       case DOWNLINK_FORMAT_LOW_RATE_CDH_BIN:
          subset->MainBusVoltage = block->telemetry.data[71];
          subset->SisInternalTemperature1  = block->telemetry.data[193];
          subset->SisInternalTemperature2  = block->telemetry.data[194];
          subset->SisInterfaceTemperature  = block->telemetry.data[336];
          subset->SisCoverTelltale =
                            (block->telemetry.data[197] & 0x40) ? 1 : 0;
          subset->SisCurrent  = block->telemetry.data[199];
          break;
       case DOWNLINK_FORMAT_LOW_RATE_ADC:
          subset->MainBusVoltage = block->telemetry.data[68];
          subset->SisInternalTemperature1 =
                                       block->telemetry.data[190];
          subset->SisInternalTemperature2 =
                                       block->telemetry.data[191];
          subset->SisCoverTelltale =
                      (block->telemetry.data[194] & 0x40) ? 1 : 0;
          subset->SisCurrent  = block->telemetry.data[196];
          subset->SisInterfaceTemperature  = 0;
          break;
      case DOWNLINK_FORMAT_RTSW:
          subset->MainBusVoltage = block->telemetry.data[810];
          subset->SisInternalTemperature1  = 0;
          subset->SisInternalTemperature2  = 0;
          subset->SisInterfaceTemperature  = 0;
          subset->SisCurrent  = 0;
          subset->SisCoverTelltale = 255;
          break;
      default:
          subset->MainBusVoltage = 0;
          subset->SisInternalTemperature1  = 0;
          subset->SisInternalTemperature2  = 0;
          subset->SisInterfaceTemperature  = 0;
          subset->SisCurrent  = 0;
          subset->SisCoverTelltale = 255;
          break;
   }

   switch (subset->FormatId)
   {
       case DOWNLINK_FORMAT_SCIENCE:
       case DOWNLINK_FORMAT_ADC:
       case DOWNLINK_FORMAT_CDH_MEMORY:
       case DOWNLINK_FORMAT_CDH_BIN:
          memcpy(subset->SisData,
                 block->telemetry.data + TELEMETRY_OFFSET_SIS,
                 NUM_SIS_TELEMETRY_BYTES);
          break;
   
      default:
          memset(subset->SisData, 0, NUM_SIS_TELEMETRY_BYTES);
          break;
   }
}
